using System;

namespace SBU.HTRRL.FlowPattern.DataAnalysis
{
	/// <summary>
	/// Define the kind of properties.
	/// </summary>
	class OutOfRangeException:ApplicationException 
	{
		public OutOfRangeException():base(){}
		public OutOfRangeException(string str):base(str){}
		public override string ToString() 
		{
			return "The parameters are out of range!";
		}
	}

	class WrongParameterException:ApplicationException 
	{
		public WrongParameterException():base(){}
		public WrongParameterException(string str):base(str){}
		public override string ToString() 
		{
			return "The input parameters are wrong!";
		}
	}

	class WrongStateException:ApplicationException 
	{
		public WrongStateException():base(){}
		public WrongStateException(string str):base(str){}
		public override string ToString() 
		{
			return "The state are wrong!";
		}
	}

	public abstract class Property
	{
		protected double t; //t(K)
		protected double p; //p(Pa-a)
		protected Property(double T,double P) //T(K), P(Pa-a)
		{
			//
			// Initial temperature and temperature
			//
			t=T;
			p=P;
		}
		public abstract string State //T(K)
		{
			get;
		}
		public abstract double SaturatedTemperature //T(K)
		{
			get;
		}
		public abstract double SaturatedPressure //P(Pa-a)
		{
			get;
		}

		public abstract double LiquidDensity //Q(kg/m3)
		{
			get;
		}

		public abstract double VapourDensity //Q(kg/m3)
		{
			get;
		}
		public abstract double LiquidEnthalpy //H(J/kg)
		{
			get;
		}
		public abstract double VapourEnthalpy //H(J/kg)
		{
			get;
		}
		public abstract double LatentHeat //Hfg(J/kg)
		{
			get;
		}
		public abstract double SaturatedLiquidViscosity //Mu(Pa.s)
		{
			get;
		}
		public abstract double SaturatedVapourViscosity //Mu(Pa.s)
		{
			get;
		}
		public abstract double SurfaceTension //Mu(N/m)
		{
			get;
		}
	}

	public class R134aProperty:Property
	{
		const double Tc=374.3, Pc=4064000.0,Qc=508.0,R=81.4892;
		const double satTempRange=0.2;
		static double[,] a = {
					{0,0,0,0,0,0},
					{0,0,0,0,0,0},
					{0,10.200664,-33.117138,27.006031,-9.3442006,0.043959113},
					{0,-32.533383,84.321026,-53.638675,5.1293252,0},
					{0,78.123563,-186.15733,105.08815,0,0},
					{0,-68.597038,163.7985,-89.655243,0,0},
					{0,28.411548,-68.88115,34.766061,0,0},
					{0,-4.9177332,13.40429,-5.7021512,0,0},
					{0,0,-0.61139649,0,0,0},
					{0,0.057702463,-0.077630849,0.069571809,0,0}
							 };
		public readonly double Temperature, Pressure;

		public R134aProperty(double T, double P):base(T,P)
		{
			Temperature=T;
			Pressure=P;
		}

		public override string State //liquid,two-phase, vapour
		{
			get
			{
				string[] state={"liquid","two-phase","vapour"};
				string strState;
				if (t<(this.SaturatedTemperature-satTempRange))
					strState=state[0];
				else if (t>(this.SaturatedTemperature+satTempRange))
					strState=state[2];
				else
					strState=state[1];
				return strState;
			}
		}

		public override double SaturatedTemperature // Range: P=0.1417-40.64 bar
		{
			get
			{
				double Psr, T0, Tr,Fx,dFx, Ts;
				try
				{
					if (p<14170||p>4064000)
					{
						throw new OutOfRangeException();
					}
					Psr = p / Pc;
					T0 = 246.97;
					Tr = T0 / Tc;
					do
					{
						Fx = (-7.651 * (1 - Tr) + 1.8458 * Math.Pow((1 - Tr),1.5) - 0.6821 * Math.Pow((1 - Tr),2) - 3.059 * Math.Pow((1 - Tr),3)) / Tr - Math.Log(Psr);
						dFx = -9.8591 + 6.118 * Tr + (11.3921 - 2.7687 * Tr * Math.Pow((1 - Tr),0.5) - 1.8458 * Math.Pow((1 - Tr), 1.5)) / Math.Pow(Tr,2);
						Tr = Tr - Fx / dFx;
					}while (Math.Abs(Fx) > 0.0001);
					Ts = Tr * Tc;
					return Ts;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double SaturatedPressure //Range: T=-62.15-101.15 C
		{
			get
			{
				double k, Tr, Ps;
				try
				{
					if (t<211||t>374.3)
						throw new OutOfRangeException();            
					Tr = t / Tc;
					k = 1 - Tr;
					Ps =1.0 / Tr * (-7.651 * k + 1.8458 * Math.Pow(k, 1.5) - 0.6821 * Math.Pow(k, 2.0) - 3.059 * Math.Pow(k, 3.0));
					Ps = Math.Exp(Ps) * Pc;
					return Ps;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double LiquidDensity
			//Range: T=-73.15-101.15 C
			//saturated liquid density, avaulable in the current experimetal range with limited subcooling degree T=-20 C-sat. P=1-20 bar, error<0.5%
		{
			get
			{
				double k,Tr,Qr,Q;
				try
				{
					if (this.State=="vapour")
						throw new WrongStateException();
					if (t<200||t>=374.3)
						throw new OutOfRangeException();
					Tr = t / Tc;
					k = 1 - Tr;
					Qr = 1.0 + 2.451 * Math.Pow(k,0.38) + 0.4403 * Math.Pow(k,1.6);
					Q = Qr * Qc;
					return Q;
				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double VapourDensity
			//Range(vapour): T=-33.15-206.85 C P=0-40.64 bar
			//calculate supheated/saturated vapour or high pressure subcooling liquid density
		{
			get
			{
				double Zc,Tr,Pr,Q,Qr,Pr1;
				try
				{
					if (this.State=="liquid")
						throw new WrongStateException();
					if (t<240||t>480||p<0||p>4064000)
						throw new OutOfRangeException();
					Zc = Pc / (R * Qc * Tc);
					Tr = t / Tc;
					Pr = p / Pc;
					Q = 0;
					do
					{
						Qr = Q / Qc;
						Pr1 = Tr * Qr / Zc;
						for (int i = 2;i<=9;i++) 
						{
							Pr1 = Pr1 + (a[i, 1] + a[i, 2] / Tr + a[i, 3] / Math.Pow(Tr,2) + a[i, 4] / Math.Pow(Tr,3)+ a[i, 5] / Math.Pow(Tr,5)) * Math.Pow(Qr,i);
						}
						Q = Q + (Pr - Pr1);
					}
					while (Math.Abs(Pr - Pr1) > 0.00001);
					return Q;
				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double LiquidEnthalpy
			//Range(vapour): T=-33.15-206.85 C P=0-150 bar
			//specific enthalpy (Inc. subcooling/saturated/superheated vapour/liquid)
		{
			get
			{
				double [] c={0,2.01832,12.891165,-3.83066,0.71466,0};
				double Cs = 17.784918, Cf = 0.4043836,fr = 0.0,H1 = 0.0,H2 = 0.0;
				double Tr,Qr,Pr,Zc,Tr0,H3;
				try
				{
					if (this.State=="vapour")
						throw new WrongStateException();
					if (t<240||t>480||p<0||p>15000000)
						throw new OutOfRangeException();
					Tr = t / Tc;
					Pr = t / Pc;
					Qr = this.LiquidDensity / Qc;
					Zc = Pc / (R * Qc * Tc);
					Tr0 = 273.15 / Tc;
					for (int i = 2; i<=9;i++)
					{	
						fr = fr + (a[i, 1] + a[i, 2] / Tr + a[i, 3] / Math.Pow(Tr,2) + a[i, 4] / Math.Pow(Tr,3) + a[i, 5] / Math.Pow(Tr,5)) * (Math.Pow(Qr,(i - 1)) / (i - 1));
					}
					fr = fr + Tr / Zc * Math.Log(Qr) + ((c[1] - 1) * (Tr - Tr0 - Tr * Math.Log(Tr / Tr0)) - c[2] * Math.Pow((Tr - Tr0),2) / 2 - c[3] * (Math.Pow(Tr,3) - 3 * Math.Pow(Tr0,2) * Tr + 2 * Math.Pow(Tr0,3)) / 6 - c[4] * (Math.Pow(Tr,4) - 4 * Math.Pow(Tr0,3) * Tr + 3 * Math.Pow(Tr0,4)) / 12 - (Tr - Tr0) * Cs - Cf) / Zc;
					for (int i = 2; i<=9;i++)
					{
						H1 = H1 + (a[i, 2] / Math.Pow(Tr,2) + 2 * a[i, 3] / Math.Pow(Tr,3) + 3 * a[i, 4] / Math.Pow(Tr,4) + 5 * a[i, 5] / Math.Pow(Tr,6)) * (Math.Pow(Qr,(i - 1)) / (i - 1));
					}
					H1 = (H1 - Math.Log(Qr) / Zc) * Tr;
					for (int i = 2; i<=9;i++)
					{
						H2 = H2 + Math.Pow(Qr,(i - 1)) * (a[i, 1] + a[i, 2] / Tr + a[i, 3] /Math.Pow(Tr,2) + a[i, 4] / Math.Pow(Tr,3) + a[i, 5] / Math.Pow(Tr,5));
					}
					H3 = fr + H1 + H2 + Tr / Zc + Tr / Zc * ((c[1] - 1) * Math.Log(Tr / Tr0) + c[2] * (Tr - Tr0) + c[3] * (Math.Pow(Tr,2) - Math.Pow(Tr0,2)) / 2 + c[4] * (Math.Pow(Tr,3) - Math.Pow(Tr0,3)) / 3 + Cs);
					H3 = H3 * Tc * R * Zc;
					return H3;
				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double VapourEnthalpy
			//Range(vapour): T=-33.15-206.85 C P=0-150 bar
			//specific enthalpy (Inc. subcooling/saturated/superheated vapour/liquid)
		{
			get
			{
				double [] c={0,2.01832,12.891165,-3.83066,0.71466,0};
				double Cs = 17.784918, Cf = 0.4043836,fr = 0.0,H1 = 0.0,H2 = 0.0;
				double Tr,Qr,Pr,Zc,Tr0,H3;
				try
				{
					if (this.State=="liquid")
						throw new WrongStateException();
					if (t<240||t>480||p<0||p>15000000)
						throw new OutOfRangeException();
					Tr = t / Tc;
					Pr = p / Pc;
					Qr = this.VapourDensity / Qc;
					Zc = Pc / (R * Qc * Tc);
					Tr0 = 273.15 / Tc;
					for (int i = 2; i<=9;i++)
					{	
						fr = fr + (a[i, 1] + a[i, 2] / Tr + a[i, 3] / Math.Pow(Tr,2) + a[i, 4] / Math.Pow(Tr,3) + a[i, 5] / Math.Pow(Tr,5)) * (Math.Pow(Qr,(i - 1)) / (i - 1));
					}
					fr = fr + Tr / Zc * Math.Log(Qr) + ((c[1] - 1) * (Tr - Tr0 - Tr * Math.Log(Tr / Tr0)) - c[2] * Math.Pow((Tr - Tr0),2) / 2 - c[3] * (Math.Pow(Tr,3) - 3 * Math.Pow(Tr0,2) * Tr + 2 * Math.Pow(Tr0,3)) / 6 - c[4] * (Math.Pow(Tr,4) - 4 * Math.Pow(Tr0,3) * Tr + 3 * Math.Pow(Tr0,4)) / 12 - (Tr - Tr0) * Cs - Cf) / Zc;
					for (int i = 2; i<=9;i++)
					{
						H1 = H1 + (a[i, 2] / Math.Pow(Tr,2) + 2 * a[i, 3] / Math.Pow(Tr,3) + 3 * a[i, 4] / Math.Pow(Tr,4) + 5 * a[i, 5] / Math.Pow(Tr,6)) * (Math.Pow(Qr,(i - 1)) / (i - 1));
					}
					H1 = (H1 - Math.Log(Qr) / Zc) * Tr;
					for (int i = 2; i<=9;i++)
					{
						H2 = H2 + Math.Pow(Qr,(i - 1)) * (a[i, 1] + a[i, 2] / Tr + a[i, 3] /Math.Pow(Tr,2) + a[i, 4] / Math.Pow(Tr,3) + a[i, 5] / Math.Pow(Tr,5));
					}
					H3 = fr + H1 + H2 + Tr / Zc + Tr / Zc * ((c[1] - 1) * Math.Log(Tr / Tr0) + c[2] * (Tr - Tr0) + c[3] * (Math.Pow(Tr,2) - Math.Pow(Tr0,2)) / 2 + c[4] * (Math.Pow(Tr,3) - Math.Pow(Tr0,3)) / 3 + Cs);
					H3 = H3 * Tc * R * Zc;
					return H3;
				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double LatentHeat
		{
			get
			{
				try
				{
					if (this.State!="two-phase")
						throw new WrongStateException();
					if (this.LiquidEnthalpy==-999||this.VapourEnthalpy==-999)
						throw new OutOfRangeException();
					return this.VapourEnthalpy-this.LiquidEnthalpy;
				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double SaturatedLiquidViscosity
			// saturated liquid viscosity
			// Range: T=-22.2-70 C
		{
			get
			{
				double Mu;
				try
				{
					if (this.State!="two-phase")
						throw new WrongStateException();
					if (t<250.95||t>343.15)
						throw new OutOfRangeException();			
					Mu = -6.0481 + 906.845 / t + 0.0109872 * t - 0.0000210309 * Math.Pow(t,2);
					Mu = Math.Exp(Mu) * 0.001;
					return Mu;
				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double SaturatedVapourViscosity
			//saturated vapour viscosity
			//Range: T=-40-80 C
		{
			get
			{
				double Mu;
				try
				{
					if (this.State!="two-phase")
						throw new WrongStateException();
					if (t<233.15||t>353.15)
						throw new OutOfRangeException();
					Mu = 4.3045 * Math.Pow(t,5) / Math.Pow(10,16) - 5.9486 * Math.Pow(t,4) / Math.Pow(10,13) + 3.2918 * Math.Pow(t,3) /Math.Pow(10,10) - 9.1102 * Math.Pow(t,2) / Math.Pow(10,8) + 1.2641 * t / Math.Pow(10,5) - 6.9666 / 10000;
					return Mu;				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}

		public override double SurfaceTension
			//surface tension
			//Range: T=-43.15-101.15 C
		{
			get
			{
				double Tr,S;
				try
				{
					if (this.State!="two-phase")
						throw new WrongStateException();
					if (t<230||t>374.3)
						throw new OutOfRangeException();
					Tr = t / Tc;
					S = 0.001 * 55.44 * Math.Pow((1.0 - Tr),1.22);
					return S;
				}
				catch (WrongStateException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
				catch (OutOfRangeException exc)
				{
					Console.WriteLine(exc);
					return -999;
				}
			}
		}
	}	

	public class R134aPropertyUncertainty
	{
		private double Temperature, Pressure, E_T, E_P;
		private const double dT=0.01, dP=100.0;
		string state;
		private R134aProperty R134a, R134a_dT, R134a_dP;

		public R134aPropertyUncertainty(double T, double P,double Error_T, double Error_P)
		{
			Temperature=T;
			Pressure=P;
			E_T=Error_T;
			E_P=Error_P;
			R134a=new R134aProperty(T,P);
			R134a_dT=new R134aProperty(T+dT,P);
			R134a_dP=new R134aProperty(T,P+dP);
			state=this.R134a.State;
			if (state=="two-phase")
			{
				R134a_dT=new R134aProperty(T+dT,P+this.SaturatedPressure_TemperatureDerivative*dT);
				R134a_dP=new R134aProperty(T+this.SaturatedTemperature_PressureDerivative*dP,P+dP);
			}
		}

		public double SaturatedTemperature_PressureDerivative
		{
			get
			{
				double dTs_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dTs_dP=(this.R134a_dP.SaturatedTemperature-this.R134a.SaturatedTemperature)/dP;
						break;
					}
				}
				return dTs_dP;
			}
		}

		public double SaturatedPressure_TemperatureDerivative
		{
			get
			{
				double dPs_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dPs_dT=(this.R134a_dT.SaturatedPressure-this.R134a.SaturatedPressure)/dT;
						break;
					}
				}
				return dPs_dT;
			}
		}

		public double LiquidDensity_TemperatureDerivative
		{
			get
			{
				double dRol_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						dRol_dT=(this.R134a_dT.LiquidDensity-this.R134a.LiquidDensity)/dT;
						break;
					}
					case "two-phase":
					{
						dRol_dT=(this.R134a_dT.LiquidDensity-this.R134a.LiquidDensity)/dT;
						break;
					}
				}
				return dRol_dT;
			}
		}

		public double LiquidDensity_PressureDerivative
		{
			get
			{
				double dRol_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						dRol_dP=(this.R134a_dP.LiquidDensity-this.R134a.LiquidDensity)/dP;
						break;
					}
					case "two-phase":
					{
						dRol_dP=(this.R134a_dP.LiquidDensity-this.R134a.LiquidDensity)/dP;
						break;
					}
				}
				return dRol_dP;
			}
		}

		public double VapourDensity_TemperatureDerivative
		{
			get
			{
				double dRov_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dRov_dT=(this.R134a_dT.VapourDensity-this.R134a.VapourDensity)/dT;
						break;
					}
				}
				return dRov_dT;
			}
		}

		public double VapourDensity_PressureDerivative
		{
			get
			{
				double dRov_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dRov_dP=(this.R134a_dP.VapourDensity-this.R134a.VapourDensity)/dP;
						break;
					}
				}
				return dRov_dP;
			}
		}

		public double LiquidEnthalpy_TemperatureDerivative
		{
			get
			{
				double dHl_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						dHl_dT=(this.R134a_dT.LiquidEnthalpy-this.R134a.LiquidEnthalpy)/dT;
						break;
					}
					case "two-phase":
					{
						dHl_dT=(this.R134a_dT.LiquidEnthalpy-this.R134a.LiquidEnthalpy)/dT;
						break;
					}
				}
				return dHl_dT;
			}
		}

		public double LiquidEnthalpy_PressureDerivative
		{
			get
			{
				double dHl_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						dHl_dP=(this.R134a_dP.LiquidEnthalpy-this.R134a.LiquidEnthalpy)/dP;
						break;
					}
					case "two-phase":
					{
						dHl_dP=(this.R134a_dP.LiquidEnthalpy-this.R134a.LiquidEnthalpy)/dP;
						break;
					}
				}
				return dHl_dP;
			}
		}

		public double VapourEnthalpy_TemperatureDerivative
		{
			get
			{
				double dHv_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dHv_dT=(this.R134a_dT.VapourEnthalpy-this.R134a.VapourEnthalpy)/dT;
						break;
					}
				}
				return dHv_dT;
			}
		}

		public double VapourEnthalpy_PressureDerivative
		{
			get
			{
				double dHv_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dHv_dP=(this.R134a_dP.VapourEnthalpy-this.R134a.VapourEnthalpy)/dP;
						break;
					}
				}
				return dHv_dP;
			}
		}

		public double LatentHeat_PressureDerivative
		{
			get
			{
				double dHfg_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dHfg_dP=(this.R134a_dP.LatentHeat-this.R134a.LatentHeat)/dP;
						break;
					}
				}
				return dHfg_dP;
			}
		}

		public double LatentHeat_TemperatureDerivative
		{
			get
			{
				double dHfg_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dHfg_dT=(this.R134a_dT.LatentHeat-this.R134a.LatentHeat)/dT;
						break;
					}
				}
				return dHfg_dT;
			}
		}

		public double SaturatedLiquidViscosity_TemperatureDerivative
		{
			get
			{
				double dMul_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dMul_dT=(this.R134a_dT.SaturatedLiquidViscosity-this.R134a.SaturatedLiquidViscosity)/dT;
						break;
					}
				}
				return dMul_dT;
			}
		}

		public double SaturatedLiquidViscosity_PressureDerivative
		{
			get
			{
				double dMul_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dMul_dP=(this.R134a_dP.SaturatedLiquidViscosity-this.R134a.SaturatedLiquidViscosity)/dP;
						break;
					}
				}
				return dMul_dP;
			}
		}

		public double SaturatedVapourViscosity_TemperatureDerivative
		{
			get
			{
				double dMuv_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dMuv_dT=(this.R134a_dT.SaturatedVapourViscosity-this.R134a.SaturatedVapourViscosity)/dT;
						break;
					}
				}
				return dMuv_dT;
			}
		}

		public double SaturatedVapourViscosity_PressureDerivative
		{
			get
			{
				double dMuv_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dMuv_dP=(this.R134a_dP.SaturatedVapourViscosity-this.R134a.SaturatedVapourViscosity)/dP;
						break;
					}
				}
				return dMuv_dP;
			}
		}

		public double SurfaceTension_TemperatureDerivative
		{
			get
			{
				double dSt_dT=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dSt_dT=(this.R134a_dT.SurfaceTension-this.R134a.SurfaceTension)/dT;
						break;
					}
				}
				return dSt_dT;
			}
		}

		public double SurfaceTension_PressureDerivative
		{
			get
			{
				double dSt_dP=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						dSt_dP=(this.R134a_dP.SurfaceTension-this.R134a.SurfaceTension)/dP;
						break;
					}
				}
				return dSt_dP;
			}
		}

		public double SaturatedTemperature_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_Ts=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_Ts=Math.Abs(this.SaturatedTemperature_PressureDerivative*E_P);
						break;
					}
				}
				return E_Ts;
			}
		}

		public double SaturatedPressure_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_Ps=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_Ps=Math.Abs(E_P);
						break;
					}
				}
				return E_Ps;
			}
		}

		public double LiquidDensity_Error //absolute error due to the error of pressure measurement (two-phase) or the error of temperature and pressure measurements (liquid)
		{
			get
			{
				double E_Rol=-999;
				switch (state)
				{
					case "liquid":
					{
						E_Rol=Math.Sqrt(Math.Pow(this.LiquidDensity_PressureDerivative*E_P,2)+Math.Pow(this.LiquidDensity_TemperatureDerivative*E_T,2));
						break;
					}
					case "two-phase":
					{
						E_Rol=Math.Abs(this.LiquidDensity_PressureDerivative*E_P);
						break;
					}
				}
				return E_Rol;
			}
		}

		public double VapourDensity_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_Rov=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_Rov=Math.Abs(this.VapourDensity_PressureDerivative*E_P);
						break;
					}
				}
				return E_Rov;
			}
		}

		public double LiquidEnthalpy_Error //absolute error due to the error of pressure measurement (two-phase) or the error of temperature and pressure measurements (liquid)
		{
			get
			{
				double E_Hl=-999;
				switch (state)
				{
					case "liquid":
					{
						E_Hl=Math.Sqrt(Math.Pow(this.LiquidEnthalpy_PressureDerivative*E_P,2)+Math.Pow(this.LiquidEnthalpy_TemperatureDerivative*E_T,2));
						break;
					}
					case "two-phase":
					{
						E_Hl=Math.Abs(this.LiquidEnthalpy_PressureDerivative*E_P);
						break;
					}
				}
				return E_Hl;
			}
		}

		public double VapourEnthalpy_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_Hv=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_Hv=Math.Abs(this.VapourEnthalpy_PressureDerivative*E_P);
						break;
					}
				}
				return E_Hv;
			}
		}

		public double LatentHeat_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_Hfg=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_Hfg=Math.Abs(this.LatentHeat_PressureDerivative*E_P);
						break;
					}
				}
				return E_Hfg;
			}
		}

		public double SaturatedLiquidViscosity_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_Mul=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_Mul=Math.Abs(this.SaturatedLiquidViscosity_PressureDerivative*E_P);
						break;
					}
				}
				return E_Mul;
			}
		}

		public double SaturatedVapourViscosity_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_Muv=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_Muv=Math.Abs(this.SaturatedVapourViscosity_PressureDerivative*E_P);
						break;
					}
				}
				return E_Muv;
			}
		}

		public double SurfaceTension_Error //absolute error due to the error of pressure measurement
		{
			get
			{
				double E_St=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						E_St=Math.Abs(this.SurfaceTension_PressureDerivative*E_P);
						break;
					}
				}
				return E_St;
			}
		}

		public double SaturatedTemperature_Uncertainty //relative error
		{
			get
			{
				double U_Ts=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_Ts=this.SaturatedTemperature_Error/R134a.SaturatedTemperature;
						break;
					}
				}
				return U_Ts;
			}
		}

		public double SaturatedPressure_Uncertainty //relative error
		{
			get
			{
				double U_Ps=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_Ps=this.SaturatedPressure_Error/R134a.SaturatedPressure;
						break;
					}
				}
				return U_Ps;
			}
		}

		public double LiquidDensity_Uncertainty //relative error
		{
			get
			{
				double U_Rol=-999;
				switch (state)
				{
					case "liquid":
					{
						U_Rol=this.LiquidDensity_Error/R134a.LiquidDensity;
						break;
					}
					case "two-phase":
					{
						U_Rol=this.LiquidDensity_Error/R134a.LiquidDensity;
						break;
					}
				}
				return U_Rol;
			}
		}

		public double VapourDensity_Uncertainty //relative error
		{
			get
			{
				double U_Rov=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_Rov=this.VapourDensity_Error/R134a.VapourDensity;
						break;
					}
				}
				return U_Rov;
			}
		}

		public double LiquidEnthalpy_Uncertainty //relative error
		{
			get
			{
				double U_Hl=-999;
				switch (state)
				{
					case "liquid":
					{
						U_Hl=this.LiquidEnthalpy_Error/R134a.LiquidEnthalpy;
						break;
					}
					case "two-phase":
					{
						U_Hl=this.LiquidEnthalpy_Error/R134a.LiquidEnthalpy;
						break;
					}
				}
				return U_Hl;
			}
		}

		public double VapourEnthalpy_Uncertainty //relative error
		{
			get
			{
				double U_Hv=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_Hv=this.VapourEnthalpy_Error/R134a.VapourEnthalpy;
						break;
					}
				}
				return U_Hv;
			}
		}

		public double LatentHeat_Uncertainty //relative error
		{
			get
			{
				double U_Hfg=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_Hfg=this.LatentHeat_Error/R134a.LatentHeat;
						break;
					}
				}
				return U_Hfg;
			}
		}

		public double SaturatedLiquidViscosity_Uncertainty //relative error
		{
			get
			{
				double U_Mul=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_Mul=this.SaturatedLiquidViscosity_Error/R134a.SaturatedLiquidViscosity;
						break;
					}
				}
				return U_Mul;
			}
		}

		public double SaturatedVapourViscosity_Uncertainty //relative error
		{
			get
			{
				double U_Muv=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_Muv=this.SaturatedVapourViscosity_Error/R134a.SaturatedVapourViscosity;
						break;
					}
				}
				return U_Muv;
			}
		}

		public double SurfaceTension_Uncertainty //relative error
		{
			get
			{
				double U_St=-999;
				switch (state)
				{
					case "liquid":
					{
						break;
					}
					case "two-phase":
					{
						U_St=this.SurfaceTension_Error/R134a.SurfaceTension;
						break;
					}
				}
				return U_St;
			}
		}
	}
}